package MyMouseA;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;

use Mouse;

has 'b' => (is => 'rw', isa => 'MyMouseB');

1;