package Bar7::Meta::Trait;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;
use Mouse::Role;

around _immutable_options => sub { };

no Mouse::Role;

1;
